#!/usr/bin/env python3
# Copyright 2019 Open Source Robotics Foundation, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import datetime
import logging
import os
import shutil
import subprocess
import sys
import tarfile

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger('run_benchmarks')


def run_benchmark(target, output_dir):
    """Run a specified benchmark target."""
    basename = os.path.basename(target)
    name = basename.split('BENCHMARK_')[1]
    filename = os.path.join(output_dir, name + '.json')

    logger.info('Running benchmark: {name}'.format_map(locals()))

    cmd = [
        target,
        '--benchmark_out={filename}'.format_map(locals()),
        '--benchmark_out_format=json'
    ]

    logger.debug(' '.join(cmd))
    try:
        benchmark = subprocess.call(cmd,
                                    stdout=sys.stdout,
                                    stderr=sys.stderr)
    except (OSError, subprocess.CalledProcessError) as exception:
        logger.error('Exception occurred: ' + str(exception))
        logger.error('Benchmark failed')
    else:
        logger.info('Finished benchmark: {name}'.format_map(locals()))


def run_benchmarks(targets, output_dir):
    """Run all benchmark targets in a list."""
    for target in targets:
        run_benchmark(target, output_dir)


def create_results_dir(project_name, time, root, version_file):
    """Create a directory to store benchmark results in."""
    time_str = time.strftime('%Y-%m-%d_%H-%M-%S')

    folder_name = '{project_name}_{time_str}'.format_map(locals())
    results_dir = os.path.join(root, folder_name)
    logger.info('Creating results dir: {results_dir}'.format_map(locals()))

    # Create output paths as needed.
    if not os.path.exists(root):
        os.mkdir(root)
    if not os.path.exists(results_dir):
        os.mkdir(results_dir)

    # Copy version information over
    shutil.copy(version_file, os.path.join(results_dir, 'version_info.json'))
    return results_dir


def collect_results(project_name, results_dir):
    """Collect results from a given execution into a tar file."""
    path, basename = os.path.split(results_dir)
    tarname = basename + '.tar.gz'

    with tarfile.open(os.path.join(path, tarname), 'w:gz') as tar:
        tar.add(results_dir, arcname=basename)
    logger.info('Benchmark results collected in: ' +
                os.path.join(path, tarname))


if __name__ == '__main__':
    parser = argparse.ArgumentParser('Run and aggregate available benchmarks')
    parser.add_argument('--project-name', help='Name of the Ignition project')
    parser.add_argument('--version-file',
                        help='Generated file containing version information')
    parser.add_argument('--benchmark-targets', help='Targets to be executed')
    parser.add_argument('--results-root',
                        help='Root directory to store results')

    args = parser.parse_args()

    # Targets will be semicolon delimited from CMake.

    if not os.path.exists(args.benchmark_targets):
        print('Invalid Targets File: {f}'.format(f=args.benchmark_targets))
        parser.exit()

    targets = []
    with open(args.benchmark_targets, 'r') as f:
        for line in f.readlines():
            targets.extend(line.split(';'))

    results_dir = create_results_dir(args.project_name,
                                     datetime.datetime.utcnow(),
                                     args.results_root,
                                     args.version_file)

    run_benchmarks(targets, results_dir)
    collect_results(args.project_name, results_dir)



#!/usr/bin/env python
# Software License Agreement (BSD License)
#
# Copyright (c) 2008, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of Willow Garage, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Revision $Id: check_test_ran.py 16671 2012-04-27 16:15:28Z dthomas $

"""
Writes a test failure out to test file if it doesn't exist.
"""

# Adapted from rosunit/check_test_ran.py

from __future__ import print_function
NAME="check_test_ran.py"

import os
import sys

def usage():
    print("""Usage:
\t%s test-file.xml
"""%(NAME), file=sys.stderr)
    print(sys.argv)
    sys.exit(getattr(os, 'EX_USAGE', 1))

def check_main():
    if len(sys.argv) < 2:
        usage()
    test_file = sys.argv[1]
        
    print("Checking for test results in %s"%test_file)
    
    if not os.path.exists(test_file):
        if not os.path.exists(os.path.dirname(test_file)):
            os.makedirs(os.path.dirname(test_file))
            
        print("Cannot find results, writing failure results to", test_file)
        
        with open(test_file, 'w') as f:
            test_name = os.path.basename(test_file)
            d = {'test': test_name, 'test_file': test_file }
            f.write("""<?xml version="1.0" encoding="UTF-8"?>
<testsuite tests="1" failures="1" time="1" errors="0" name="%(test)s">
  <testcase name="test_ran" status="run" time="1" classname="Results">
    <failure message="Unable to find test results for %(test)s, test did not run.\nExpected results in %(test_file)s" type=""/>
  </testcase>
</testsuite>"""%d)

if __name__ == '__main__':
    check_main()
